﻿using System;
using System.IO;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;

namespace Aplikace {
    public partial class Form1 : Form {
        bool m = false;// false = adresáře, true = obrázky
        bool mode {
            set {
                m = value;
                sc.Visible = !m;
                gal.Visible = m;
            }
            get {
                return m;
            }
        } 
        string currentDir = "", currentPhoto = "";
        Thread previewThread;
        delegate void AddPrev(ucPreview uc);
        ucPreview selectedPrev;
        string[] currImgs;

        
        public Form1() {
            InitializeComponent();
            sc.Dock = DockStyle.Fill;
            gal.Dock = DockStyle.Fill;
            gal.CloseGal += new EventHandler(CloseGal);
            buildTree();
        }
        /// <summary>
        /// Vytvoří počáteční strom s písmeny disků
        /// </summary>
        void buildTree(){
            string[] drives = Directory.GetLogicalDrives();
            for (int i = 0; i < drives.Length; i++) {
                TreeNode tn = new TreeNode(drives[i]);
                tn.Name = drives[i];
                twTree.Nodes.Add(tn);
                growTree(tn);
            }
            
        }
        /// <summary>
        /// rozroste stom
        /// </summary>
        void growTree(TreeNode tn) {
            if (Directory.Exists(tn.Name)) {
                try {
                    string[] arr = Directory.GetDirectories(tn.Name);
                    Array.Sort(arr);
                    tn.Collapse();
                    for (int i = 0; i < arr.Length; i++) {
                        TreeNode tnChild = new TreeNode(arr[i].Substring(arr[i].IndexOf('\\') + 1));
                        tnChild.Name = arr[i];
                        tn.Nodes.Add(tnChild);
                    }
                }
                catch { }
            }
        }

        private void twTree_BeforeExpand(object sender, TreeViewCancelEventArgs e) {
            foreach (TreeNode tn in e.Node.Nodes) { // před rozbalením naplním jednotlivé Nody
                growTree(tn); 
            }
        }

        private void twTree_AfterSelect(object sender, TreeViewEventArgs e) {
            if (e.Node != null) { // po označení zobrazím náhledy
                if (previewThread != null && previewThread.IsAlive)
                    previewThread.Abort();
                flpPreviews.Controls.Clear();
                previewThread = new Thread(new ParameterizedThreadStart(showPreviews));
                previewThread.IsBackground = true;
                previewThread.Priority = ThreadPriority.BelowNormal;
                previewThread.Start(e.Node.Name);
            }

        }
        void showPreviews(object pathObj) {
            if (Directory.Exists((string)pathObj)) {
                try {
                    currImgs = Directory.GetFiles((string)pathObj, "*.jp*g");
                    Array.Sort(currImgs);
                    for(int i = 0; i < currImgs.Length; i++){
                        FileInfo fi = new FileInfo(currImgs[i]);
                        if (fi.Exists) {
                            Image bmp = Bitmap.FromFile(currImgs[i]);
                            int newWid, newHei;
                            double ratio;
                            if (bmp.Width / 4 > bmp.Height / 3) { // širší (bude se měnit výška)
                                newWid = 128;
                                ratio = (double)bmp.Width / (double)newWid;
                                newHei = (int)(bmp.Height / ratio);
                            }
                            else { // vyšší
                                newHei = 96;
                                ratio = (double)bmp.Height / (double)newHei;
                                newWid = (int)(bmp.Width / ratio);
                            }
                            Bitmap preview = new Bitmap(newWid, newHei);
                            Graphics g = Graphics.FromImage(preview);
                            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
                            g.DrawImage(bmp, new Rectangle(0, 0, newWid, newHei));
                            AddPrev ap = new AddPrev(addUC);
                            BeginInvoke(ap, new ucPreview(preview, fi.Name + Environment.NewLine +
                                fi.LastWriteTime + Environment.NewLine +
                                bmp.Width + " x " + bmp.Height + " px"+Environment.NewLine+
                                "velikost: "+ fileSizeToStr(fi.Length), i));
                        }

                    }
                }
                catch { }
            }
        }
        void addUC(ucPreview obj) {
            obj.OnSelect += new EventHandler(ucSelect);
            obj.ShowFull += new EventHandler(showGal);
            flpPreviews.Controls.Add(obj);
        }
        void ucSelect(object sender, EventArgs ea) {
            if (selectedPrev != null)
                selectedPrev.BackClr = Color.Empty;
            selectedPrev = (ucPreview)sender;
            selectedPrev.BackClr = Color.DarkGray;
        }
        void showGal(object sender, EventArgs ea) {
            mode = true;
            gal.init(currImgs);
            gal.GoToImg((int)((ucPreview)sender).Tag);
        }

        string fileSizeToStr(long bytes) {
            string size = "0 Bytů";
            if (bytes >= 1048576)
                size = String.Format("{0:##.##}", bytes / 1048576) + " MB";
            else if (bytes >= 1024)
                size = String.Format("{0:##.##}", bytes / 1024) + " KB";
            else if (bytes > 0 && bytes < 1024)
                size = bytes.ToString() + " B";
            return size;
        }
        void CloseGal(object sender, EventArgs ea) {
            mode = false;
        }

        private void Form1_KeyPress(object sender, KeyPressEventArgs e) {
            switch ((int)e.KeyChar) {
                case 27: Application.Exit(); break;
                case 42: gal.btnAuto_Click(this, new EventArgs()); break;
                case 43: gal.btnPlus_Click(this, new EventArgs()); break;
                case 45: gal.btnMinus_Click(this, new EventArgs()); break;
                default:
                    break;
            }
        }
    }
}
